# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.utils.functional import SimpleLazyObject
from django_geoip.models import IpRange
from django_geoip.utils import get_class


location_model = SimpleLazyObject(
    lambda: get_class(settings.GEOIP_LOCATION_MODEL))

storage_class = get_class(settings.GEOIP_STORAGE_CLASS)


class Locator(object):
    """ A helper class that automates user location detection
    """

    def __init__(self, request):
        self.request = request

    def locate(self):
        """ Find out what is user location (either from his ip or cookie)

        :return: Custom location model
        """
        stored_location = self._get_stored_location()
        if not stored_location:
            ip_range = self._get_ip_range()
            stored_location = self._get_corresponding_location(ip_range)
        return stored_location

    def _get_corresponding_location(self, ip_range):
        """
        Get user location by IP range, if no location matches, returns default location.

        :param ip_range: An instance of IpRange model
        :type ip_range: IpRange
        :return: Custom location model
        """
        try:
            return location_model.get_by_ip_range(ip_range)
        except ObjectDoesNotExist:
            return location_model.get_default_location()

    def _get_real_ip(self):
        """
        Get IP from request

        :param request: A usual request object
        :type request: HttpRequest
        :return: ipv4 string or None
        """
        try:
            # Trying to work with most common proxy headers
            real_ip = self.request.META['HTTP_X_FORWARDED_FOR']
            return real_ip.split(',')[0]
        except KeyError:
            return self.request.META['REMOTE_ADDR']
        except Exception:
            # Unknown IP
            return None

    def _get_ip_range(self):
        """
        Fetches IpRange instance if request IP is found in database

        :param request: A ususal request object
        :type request: HttpRequest
        :return: IpRange object or None
        """
        ip = self._get_real_ip()
        try:
            geobase_entry = IpRange.objects.by_ip(ip)
        except IpRange.DoesNotExist:
            geobase_entry = None
        return geobase_entry

    def _get_stored_location(self):
        """ Get location from cookie

        :param request: A ususal request object
        :type request: HttpRequest
        :return: Custom location model
        """
        location_storage = storage_class(request=self.request, response=None)
        return location_storage.get()




import sys

try:
    from django.test.client import RequestFactory as my_RequestFactory
except ImportError:
    my_RequestFactory = None
RequestFactory = my_RequestFactory

try:
    import unittest2 as my_unittest
except ImportError:
    if sys.version_info >= (2,7):
        # unittest2 features are native in Python 2.7
        import unittest as my_unittest
    else:
        raise
unittest = my_unittest

from test_management import *
from test_base import *
from test_storage import *
from test_models import *
from test_utils import *
from test_middleware import *
from test_views import *


# -*- coding: utf-8 -*-
from datetime import datetime
from django.conf import settings
from django.http import HttpResponse, HttpRequest
from mock import patch, Mock
from django_geoip.storage import LocationCookieStorage, LocationDummyStorage
from django_geoip.tests import unittest


class LocationCookieStorageTest(unittest.TestCase):

    def setUp(self):
        self.request = HttpRequest()
        self.request.location = Mock()

    def test_should_not_update_cookie_if_no_location_in_request(self):
        storage = LocationCookieStorage(request=HttpRequest(), response=HttpResponse())
        self.assertFalse(storage._should_update_cookie(new_value=10))

    def test_should_update_cookie_if_cookie_doesnt_exist(self):
        storage = LocationCookieStorage(request=self.request, response=HttpResponse())
        self.assertTrue(storage._should_update_cookie(new_value=10))

    def test_should_not_update_cookie_if_cookie_is_none(self):
        self.request.COOKIES[settings.GEOIP_COOKIE_NAME] = None
        storage = LocationCookieStorage(request=self.request, response=HttpResponse())
        self.assertFalse(storage._should_update_cookie(new_value=None))

    def test_should_not_update_cookie_if_cookie_is_fresh(self):
        self.request.COOKIES[settings.GEOIP_COOKIE_NAME] = 10
        storage = LocationCookieStorage(request=self.request, response=HttpResponse())
        self.assertFalse(storage._should_update_cookie(new_value=10))

    def test_should_update_cookie_if_cookie_is_obsolete(self):
        self.request.COOKIES[settings.GEOIP_COOKIE_NAME] = 42
        storage = LocationCookieStorage(request=self.request, response=HttpResponse())
        self.assertTrue(storage._should_update_cookie(new_value=10))

    @patch('django_geoip.storage.datetime')
    def test_do_set(self, mock):
        mock.now.return_value = datetime(2012, 1, 1, 0, 0, 0)
        base_response = HttpResponse()
        storage = LocationCookieStorage(request=self.request, response=base_response)
        storage._do_set(10)
        expected = 'Set-Cookie: geoip_location_id=10; expires=Tue, 20-Nov-2328 17:46:39 GMT;'
        self.assertTrue(base_response.cookies[settings.GEOIP_COOKIE_NAME].output().startswith(expected))


class LocationDummyStorageTest(unittest.TestCase):

    def setUp(self):
        self.request = HttpRequest()
        self.request.location = Mock()

    def test_get(self):
        storage = LocationDummyStorage(request=self.request, response=HttpResponse())
        self.assertEqual(storage.get(), self.request.location)

    def test_set(self):
        storage = LocationDummyStorage(request=self.request, response=HttpResponse())
        fake_location = Mock()
        storage.set(fake_location)




# -*- coding: utf-8 -*-
from decimal import Decimal
import os
import urllib2
from django.conf import settings
from django_geoip.management.commands.ipgeobase_update import Command
from django_geoip.models import City, Region, Country, IpRange
from mock import patch
from django_geoip.tests import unittest

TEST_STATIC_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), 'static'))

class DowloadTest(unittest.TestCase):
    IPGEOBASE_ZIP_FILE_PATH = 'tests.zip'
    IPGEOBASE_MOCK_URL = 'http://futurecolors/mock.zip'

    @patch('urllib2.urlopen')
    def test_download_unpack(self, mock):
        self.opener = mock.return_value
        self.opener.read.return_value = open(os.path.join(TEST_STATIC_DIR, self.IPGEOBASE_ZIP_FILE_PATH)).read()

        result = Command()._download_extract_archive(url=self.IPGEOBASE_MOCK_URL)

        mock.assert_called_once_with(self.IPGEOBASE_MOCK_URL)
        self.opener.read.assert_called_once_with()
        self.assertEqual(len(result), 2)
        self.assertTrue(result['cities'].endswith(settings.IPGEOBASE_CITIES_FILENAME))
        self.assertTrue(result['cidr'].endswith(settings.IPGEOBASE_CIDR_FILENAME))

    @patch('urllib2.urlopen')
    def test_download_exception(self, mock):
        mock.side_effect = urllib2.URLError('Response timeout')
        self.assertRaises(urllib2.URLError, Command()._download_extract_archive, self.IPGEOBASE_MOCK_URL)


class ConvertTest(unittest.TestCase):
    maxDiff = None

    def test_convert_fileline_to_dict(self):
        check_against_dict = {'city_id': u'1',
                              'city_name': u'Хмельницкий',
                              'region_name': u'Хмельницкая область',
                              'district_name': u'Центральная Украина',
                              'longitude': u'49.416668',
                              'latitude': u'27.000000'
        }

        command = Command()
        generator = command._line_to_dict(file=open(os.path.join(TEST_STATIC_DIR, 'cities.txt')),
            field_names=settings.IPGEOBASE_CITIES_FIELDS)
        result = generator.next()
        self.assertEqual(result, check_against_dict)

    def test_process_cidr_file(self):
        check_against = {
            'cidr': [
                    {'start_ip': '33554432', 'end_ip': '34603007','country_id': 'FR', 'city_id': None},
                    {'start_ip': '37249024', 'end_ip': '37251071','country_id': 'UA', 'city_id': '1'},
                    {'start_ip': '37355520', 'end_ip': '37392639','country_id': 'RU', 'city_id': '2176'},
            ],
            'countries': set([u'FR', u'UA', u'RU']),
            'city_country_mapping': {'2176': u'RU', '1': u'UA'}
        }
        command = Command()
        cidr_info = command._process_cidr_file(open(os.path.join(TEST_STATIC_DIR, 'cidr_optim.txt')))

        self.assertEqual(cidr_info['city_country_mapping'], check_against['city_country_mapping'])
        self.assertEqual(cidr_info['countries'], check_against['countries'])
        self.assertEqual(cidr_info['cidr'], check_against['cidr'])


    def test_process_cities_file(self):
        city_country_mapping = {'1': u'UA', '1057': u'RU', '2176': u'RU'}

        check_against = {
            'cities': [
                    {'region__name': u'Хмельницкая область', 'name': u'Хмельницкий',
                     'id': u'1', 'longitude': Decimal('49.416668'), 'latitude': Decimal('27.000000')},
                    {'region__name': u'Кемеровская область', 'name': u'Березовский',
                     'id': u'1057', 'longitude': Decimal('55.572479'), 'latitude': Decimal('86.192734')},
                    {'region__name': u'Ханты-Мансийский автономный округ', 'name': u'Мегион',
                     'id': u'2176', 'longitude': Decimal('61.050400'), 'latitude': Decimal('76.113472')},
            ],
            'regions': [
                    {'name':  u'Хмельницкая область', 'country__code': u'UA'},
                    {'name':  u'Кемеровская область', 'country__code': u'RU'},
                    {'name':  u'Ханты-Мансийский автономный округ', 'country__code': u'RU'},
            ]
        }

        command = Command()
        cities_info = command._process_cities_file(open(os.path.join(TEST_STATIC_DIR, 'cities.txt')), city_country_mapping)

        self.assertEqual(cities_info['cities'], check_against['cities'])
        self.assertEqual(cities_info['regions'], check_against['regions'])


class IpGeoBaseTest(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        self.countries = set([u'FR', u'UA', u'RU'])
        self.regions = [{'name': u'Хмельницкая область', 'country__code': u'UA'},
                {'name': u'Кемеровская область', 'country__code': u'RU'},
                {'name': u'Ханты-Мансийский автономный округ', 'country__code': u'RU'}, ]
        self.cities = [{'region__name': u'Хмельницкая область', 'name': u'Хмельницкий', 'id': 1},
                {'region__name': u'Кемеровская область', 'name': u'Березовский', 'id': 1057},
                {'region__name': u'Кемеровская область', 'name': u'Кемерово', 'id': 1058},
                {'region__name': u'Ханты-Мансийский автономный округ', 'name': u'Мегион', 'id': 2176}, ]
        self.cidr = {
            'cidr': [
                    {'start_ip': '33554432', 'end_ip': '34603007','country_id': 'FR', 'city_id': None},
                    {'start_ip': '37249024', 'end_ip': '37251071','country_id': 'UA', 'city_id': '1'},
                    {'start_ip': '37355520', 'end_ip': '37392639','country_id': 'RU', 'city_id': '2176'},
            ],
            'countries': set([u'FR', u'UA', u'RU']),
            'city_country_mapping': {2176: u'RU', 1058: u'RU', 1057: u'RU', 1: u'UA'}
        }
        City.objects.all().delete()
        Region.objects.all().delete()
        Country.objects.all().delete()

    def test_update_geography_empty_data(self):
        command = Command()
        cities_info = command._update_geography(self.countries, self.regions, self.cities)

        self.assertEqual(set(Country.objects.all().values_list('code', flat=True)), self.countries)
        self.assertEqual(list(Region.objects.all().values('name', 'country__code')), self.regions)
        self.assertEqual(list(City.objects.all().values('name', 'id', 'region__name')), self.cities)

    def test_update_pre_existing_data(self):
        self.assertTrue(Country.objects.all().count() == 0)
        ua = Country.objects.create(name=u'UA', code=u'UA')
        ru = Country.objects.create(name=u'RU', code=u'RU')

        kemerovo = Region.objects.create(name=u'Кемеровская область', country=ru)
        City.objects.create(name=u'Березовский', id=1057, region=kemerovo)

        command = Command()
        command._update_geography(self.countries, self.regions, self.cities)

        self.assertEqual(set(Country.objects.all().values_list('code', flat=True)), self.countries)
        self.assertItemsEqual(list(Region.objects.all().values('name', 'country__code')), self.regions)
        self.assertEqual(list(City.objects.all().values('name', 'id', 'region__name')), self.cities)

    def test_build_city_region_mapping(self):
        check_against_mapping = {
            1: 1,
            1057: 2,
            1058: 2,
            2176: 3,
            }
        for region in self.regions:
            Region.objects.create(name=region['name'], country_id=region['country__code'])
        for city in self.cities:
            region = Region.objects.get(name=city['region__name'])
            City.objects.create(id=city['id'], name=city['name'], region=region)

        command = Command()
        mapping = command._build_city_region_mapping()

        self.assertEqual(mapping, check_against_mapping)


    def test_update_cidr(self):
        check_against_ranges = [
                {'start_ip': 33554432, 'end_ip': 34603007, 'country_id': 'FR', 'city_id': None, 'region_id': None},
                {'start_ip': 37249024, 'end_ip': 37251071, 'country_id': 'UA', 'city_id': 1, 'region_id': 1},
                {'start_ip': 37355520, 'end_ip': 37392639, 'country_id': 'RU', 'city_id': 2176,'region_id': 3},
        ]

        command = Command()
        for region in self.regions:
            Region.objects.create(name=region['name'], country_id=region['country__code'])
        for city in self.cities:
            region = Region.objects.get(name=city['region__name'])
            City.objects.create(id=city['id'], name=city['name'], region=region)
        command._update_cidr(self.cidr)

        self.assertItemsEqual(IpRange.objects.all().values('start_ip', 'end_ip', 'country_id', 'city_id', 'region_id'),
            check_against_ranges)


# -*- coding: utf-8 -*-
from appconf import AppConf

class GeoIpConfig(AppConf):
    """ GeoIP configuration """

    # Provide a model that stores geography, specific to your application
    LOCATION_MODEL = 'django_geoip.models.GeoLocationFascade'

    STORAGE_CLASS = 'django_geoip.storage.LocationCookieStorage'

    # Cookie stores location model primary key
    COOKIE_NAME = 'geoip_location_id'

    # Cookie lifetime in seconds
    COOKIE_EXPIRES = 9999999999

    class Meta:
        prefix = 'geoip'


# -*- coding: utf-8 -*-
from cStringIO import StringIO
from optparse import make_option
import tempfile
from zipfile import ZipFile
from decimal import Decimal
from django.core.management.base import BaseCommand
from django.conf import settings
import urllib2
from django_geoip.models import City, Region, Country, IpRange

class Command(BaseCommand):
    help = 'Updates ipgeobase'
    incoming = {'countries': set(),
                'city_country_mapping': {}
    }
    option_list = BaseCommand.option_list + (
        make_option('--clear',
            action='store_true',
            default=False,
            help=u"Clear tables prior import",
        ),
    )

    def handle(self, *args, **options):
        print u'Updating ipgeobase...'
        files = self._download_extract_archive(settings.IPGEOBASE_SOURCE_URL)
        cidr_info = self._process_cidr_file(open(files['cidr'], 'r'))
        city_info = self._process_cities_file(open(files['cities'], 'r'),
                                              cidr_info['city_country_mapping'])
        if options.get("clear_db"):
            IpRange.objects.all().delete()
            City.objects.all().delete()
            Region.objects.all().delete()
            Country.objects.all().delete()
        self._update_geography(cidr_info['countries'],
            city_info['regions'],
            city_info['cities'])
        self._update_cidr(cidr_info)

    def _download_url_to_string(self, url):
        f = urllib2.urlopen(url)
        buffer = StringIO(f.read())
        f.close()
        return buffer

    def _download_extract_archive(self, url):
        """ Returns dict with 2 extracted filenames """
        try:
            temp_dir = tempfile.mkdtemp()
            archive = ZipFile(self._download_url_to_string(url))
            file_cities = archive.extract(settings.IPGEOBASE_CITIES_FILENAME, path=temp_dir)
            file_cidr = archive.extract(settings.IPGEOBASE_CIDR_FILENAME, path=temp_dir)
            return {'cities': file_cities, 'cidr': file_cidr}
        except urllib2.URLError:
            raise

    def _line_to_dict(self, file, field_names):
        """ Converts file line into dictonary """
        for line in file:
            decoded_line = line.decode(settings.IPGEOBASE_FILE_ENCODING)
            delimiter = settings.IPGEOBASE_FILE_FIELDS_DELIMITER
            yield self._extract_data_from_line(decoded_line, field_names, delimiter)

    def _extract_data_from_line(self, line, field_names=None, delimiter="\t"):
        return dict(zip(field_names, line.rstrip('\n').split(delimiter)))

    def _process_cidr_file(self, file):
        """ Iterate over ip info and extract useful data """
        data = {'cidr': list(), 'countries': set(), 'city_country_mapping': dict()}
        for cidr_info in self._line_to_dict(file, field_names=settings.IPGEOBASE_CIDR_FIELDS):
            city_id = cidr_info['city_id'] if cidr_info['city_id'] != '-' else None
            data['cidr'].append({'start_ip': cidr_info['start_ip'],
                                 'end_ip': cidr_info['end_ip'],
                                 'country_id': cidr_info['country_code'],
                                 'city_id': city_id})
            data['countries'].add(cidr_info['country_code'])
            if city_id is not None:
                data['city_country_mapping'].update({cidr_info['city_id']: cidr_info['country_code']})
        return data

    def _get_country_code_for_city(self, city_id, mapping, added_data):
        """ Get coutry code for city, if we don't know exactly, lets take last used country"""
        try:
            return mapping[city_id]
        except KeyError:
            return added_data[-1]['country__code']

    def _process_cities_file(self, file, city_country_mapping):
        """ Iterate over cities info and extract useful data """
        data = {'regions': list(), 'cities': list(), 'city_region_mapping': dict()}
        for geo_info in self._line_to_dict(file, field_names=settings.IPGEOBASE_CITIES_FIELDS):
            country_code = self._get_country_code_for_city(geo_info['city_id'], city_country_mapping, data['regions'])
            new_region = {'name': geo_info['region_name'],
                          'country__code': country_code}
            if new_region not in data['regions']:
                data['regions'].append(new_region)
            data['cities'].append({'region__name': geo_info['region_name'],
                                   'name': geo_info['city_name'],
                                   'id': geo_info['city_id'],
                                   'latitude': Decimal(geo_info['latitude']),
                                   'longitude': Decimal(geo_info['longitude'])})
        return data

    def _update_geography(self, countries, regions, cities):
        """ Update database with new countries, regions and cities """
        existing = {
            'cities': list(City.objects.values_list('id', flat=True)),
            'regions': list(Region.objects.values('name', 'country__code')),
            'countries': Country.objects.values_list('code', flat=True)
        }
        for country_code in countries:
            if country_code not in existing['countries']:
                Country.objects.create(code=country_code, name=country_code)
        for entry in regions:
            if entry not in existing['regions']:
                Region.objects.create(name=entry['name'], country_id=entry['country__code'])
        for entry in cities:
            if long(entry['id']) not in existing['cities']:
                region = Region.objects.get(name=entry['region__name'])
                City.objects.create(id=entry['id'], name=entry['name'], region=region,
                                    latitude=entry.get('latitude'), longitude=entry.get('longitude'))

    def _update_cidr(self, cidr):
        """ Rebuild IPRegion table with fresh data (old ip ranges are removed for simplicity)"""
        IpRange.objects.all().delete()
        city_region_mapping = self._build_city_region_mapping()
        for entry in cidr['cidr']:
            # skipping for country rows
            if entry['city_id']:
                entry.update({'region_id': city_region_mapping[int(entry['city_id'])]})
            IpRange.objects.create(**entry)

    def _build_city_region_mapping(self):
        cities = City.objects.values('id', 'region__id')
        city_region_mapping = {}
        for city in cities:
            if city['id']:
                city_region_mapping.update({city['id']: city['region__id']})
        return city_region_mapping



from os import path
import codecs
from setuptools import setup, find_packages

read = lambda filepath: codecs.open(filepath, 'r', 'utf-8').read()

tests_require = [
    'Django>=1.2,<1.4',
    'mock>=0.7.2',
]

setup(
    name='django-geoip',
    version='0.2.1',
    author='Ilya Baryshev',
    author_email='baryshev@gmail.com',
    packages=find_packages(exclude=("tests")),
    url='https://github.com/coagulant/django-geoip',
    license='MIT',
    description="App to figure out where your visitors are from by their IP address.",
    long_description=read(path.join(path.dirname(__file__), 'README.rst')),
    tests_require=tests_require,
    test_suite = "runtests",
    extras_require={'test': tests_require},
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Country'
        db.create_table('django_geoip_country', (
            ('code', self.gf('django.db.models.fields.CharField')(max_length=2, primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('django_geoip', ['Country'])

        # Adding model 'Region'
        db.create_table('django_geoip_region', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(related_name='regions', to=orm['django_geoip.Country'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('django_geoip', ['Region'])

        # Adding unique constraint on 'Region', fields ['country', 'name']
        db.create_unique('django_geoip_region', ['country_id', 'name'])

        # Adding model 'City'
        db.create_table('django_geoip_city', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('region', self.gf('django.db.models.fields.related.ForeignKey')(related_name='cities', to=orm['django_geoip.Region'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('latitude', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=9, decimal_places=6, blank=True)),
            ('longitude', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=9, decimal_places=6, blank=True)),
        ))
        db.send_create_signal('django_geoip', ['City'])

        # Adding unique constraint on 'City', fields ['region', 'name']
        db.create_unique('django_geoip_city', ['region_id', 'name'])

        # Adding model 'IpRange'
        db.create_table('django_geoip_iprange', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('start_ip', self.gf('django.db.models.fields.BigIntegerField')(db_index=True)),
            ('end_ip', self.gf('django.db.models.fields.BigIntegerField')(db_index=True)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_geoip.Country'])),
            ('region', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_geoip.Region'], null=True)),
            ('city', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_geoip.City'], null=True)),
        ))
        db.send_create_signal('django_geoip', ['IpRange'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'City', fields ['region', 'name']
        db.delete_unique('django_geoip_city', ['region_id', 'name'])

        # Removing unique constraint on 'Region', fields ['country', 'name']
        db.delete_unique('django_geoip_region', ['country_id', 'name'])

        # Deleting model 'Country'
        db.delete_table('django_geoip_country')

        # Deleting model 'Region'
        db.delete_table('django_geoip_region')

        # Deleting model 'City'
        db.delete_table('django_geoip_city')

        # Deleting model 'IpRange'
        db.delete_table('django_geoip_iprange')


    models = {
        'django_geoip.city': {
            'Meta': {'unique_together': "(('region', 'name'),)", 'object_name': 'City'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'latitude': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '9', 'decimal_places': '6', 'blank': 'True'}),
            'longitude': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '9', 'decimal_places': '6', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'region': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cities'", 'to': "orm['django_geoip.Region']"})
        },
        'django_geoip.country': {
            'Meta': {'object_name': 'Country'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '2', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'django_geoip.iprange': {
            'Meta': {'object_name': 'IpRange'},
            'city': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_geoip.City']", 'null': 'True'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_geoip.Country']"}),
            'end_ip': ('django.db.models.fields.BigIntegerField', [], {'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'region': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_geoip.Region']", 'null': 'True'}),
            'start_ip': ('django.db.models.fields.BigIntegerField', [], {'db_index': 'True'})
        },
        'django_geoip.region': {
            'Meta': {'unique_together': "(('country', 'name'),)", 'object_name': 'Region'},
            'country': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'regions'", 'to': "orm['django_geoip.Country']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['django_geoip']



# -*- coding: utf-8 -*-
from appconf import AppConf

class IpGeoBaseConfig(AppConf):
    # URL, where to download ipgeobase file from
    SOURCE_URL = 'http://ipgeobase.ru/files/db/Main/geo_files.zip'

    FILE_FIELDS_DELIMITER = "\t"
    FILE_ENCODING = 'windows-1251'

    CITIES_FILENAME = 'cities.txt'
    #    1	Хмельницкий	Хмельницкая область	Центральная Украина	49.416668	27.000000
    CITIES_FIELDS = ['city_id', 'city_name', 'region_name', 'district_name', 'longitude', 'latitude']

    CIDR_FILENAME = 'cidr_optim.txt'
    #    1578795008	1578827775	94.26.128.0 - 94.26.255.255	RU	2287
    CIDR_FIELDS = ['start_ip', 'end_ip', 'ip_range_human', 'country_code', 'city_id']

    class Meta:
        prefix = 'ipgeobase'


# -*- coding: utf-8 -*-
from django.db import models
from django_geoip.models import GeoLocationFascade, City

class MyCustomLocation(GeoLocationFascade):
    name = models.CharField(max_length=100)
    city = models.OneToOneField(City, related_name='my_custom_location')

    @classmethod
    def get_by_ip_range(cls, ip_range):
        return ip_range.city.my_custom_location

    @classmethod
    def get_default_location(cls):
        return cls.objects.get(pk=1)

    @classmethod
    def get_available_locations(cls):
        return cls.objects.all()

    def __repr__(self):
        return 'MyCustomLocation(id={0}, city={1})'.format(self.pk, self.city.name)


# -*- coding: utf-8 -*-
import struct
import socket
from django_any.models import any_model
from django_geoip.models import IpRange, Region, Country, City
from django_geoip.tests import unittest

class IpRangeTest(unittest.TestCase):

    def setUp(self):
        self.range_contains = any_model(IpRange, start_ip=3568355840, end_ip=3568355843)
        self.range_not_contains = any_model(IpRange, start_ip=3568355844, end_ip=3568355851)

    def test_manager(self):
        ip_range = IpRange.objects.by_ip('212.176.202.2')
        self.assertEqual(ip_range, self.range_contains)
        self.assertRaises(IpRange.DoesNotExist, IpRange.objects.by_ip, '127.0.0.1')

    def test_relations(self):
        self.country = any_model(Country)
        self.region = any_model(Region, country=self.country)
        self.city = any_model(City, region=self.region)
        range = any_model(IpRange,
                          start_ip=struct.unpack('!L', socket.inet_aton('43.123.56.0'))[0],
                          end_ip=struct.unpack('!L', socket.inet_aton('43.123.56.255'))[0],
                          city=self.city, region=self.region, country=self.country)

        ip_range = IpRange.objects.by_ip('43.123.56.12')
        self.assertEqual(ip_range.city, self.city)
        self.assertEqual(ip_range.city.region, self.region)
        self.assertEqual(ip_range.city.region.country, self.country)